<?php
/*
_gallery2.plugin.php
:tabSize=2:indentSize=2:noTabs=true:
:folding=explicit:collapseFolds=1:

Copyright (C) 2007-2008 Ian Lewis (IanLewis@member.fsf.org)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
Optionally, you may find a copy of the GNU General Public License
from http://www.fsf.org/copyleft/gpl.txt
*/

if( !defined('EVO_MAIN_INIT') ) die( 'Please, do not access this page directly.' );

//Include the g2image gettext file.
// require_once dirname(__FILE__).'/g2image/gettext.inc';

/**
 * A plugin that implements single sign-on support and other integration with
 * an embedded gallery installation. Please review the help file for details 
 * on installing and using the plugin with your gallery 2 installation.
 * @author Ian Lewis (<a href="mailto:IanLewis@member.fsf.org">IanLewis@member.fsf.org</a>)
 */
class gallery2_plugin extends Plugin {
  
  var $version = '1.5.1';
  var $group = 'authentication';
  var $code = 'evo_gallery2_auth';
  
  /*
   * Lazy rendering is applied so that we don't show in the blog entry
   * admin page.
   */
  // var $apply_rendering = 'lazy';
  
  /*
   * Give us a priority of 1 so that we have priority over other potential
   * renderers.
   */
  var $priority = 1;
  var $author = 'http://www.ianlewis.org/';
  var $help_url = 'http://manual.b2evolution.net/Plugins/gallery2_plugin';
  
  // var $admin_page = false;
  var $gallery_initialized = false;
  
  var $in_install = false;
  
  /**
   * This is the normal minimum gallery2 embedded api version. We will cheat a
   * little in the future to see if they have gallery 2.3 but this is the minimum requirement.
   */
  var $minimum_g2apiversion = array(1, 1);
  
  var $g2apiversion = null;
  
  //{{{ PluginInit()
  
  /**
   * Init
   */
  function PluginInit($params) {
    $this->name = $this->T_('Gallery2');
    $this->short_desc = $this->T_('Provides integration with Gallery2');
    $this->long_desc = $this->T_('Allows including photos from your gallery2 installation in your posts and syncing of users and session info with gallery2. This is done using the <a href="http://gallery.menalto.com/apidoc/GalleryCore/Classes/GalleryEmbed.html">GalleryEmbed</a> interface and b2evolution\'s <a href="http://doc.b2evolution.net/v-1-8/plugins/Plugin.html">Plugin</a> interface. Tested on Gallery 2.1.2 and b2evolution 1.9.2.');
    $this->enable_event('tinymce_before_init');
    
    $this->admin_page = is_admin_page();
  }//}}}
  
  //{{{ GetDependencies()
  
  function GetDependencies() {
    return array(
                "required" => array("app_min" => "1.9"),
                // b2evo has a bug where recommends are treated as required
                // if the recommended plugin is installed already.
                // "recommends" => array("plugins" => 
                //                         array(array("tinymce_plugin", "0.8"))
                //  )
                );
  }//}}}
  
  //{{{ BeforeEnable()
  /**
   * Attempts to map b2evolution and gallery users automatically. 
   *
   * If a user exists in one system but not the other then the
   * user will be added to the other system in default new users group.
   *
   * If a user login exists in both systems then the usernames are
   * simply linked and given the b2evolution password.
   * The group memberships are left as is.
   *
   * The oldest b2evolution login is attempted to be matched to the oldest
   * gallery admin login.
   *
   * The gallery single-sign-on plugin does not sync groups. b2evolution does
   * not have plugin callbacks for adding/updating/deleting groups.
   */
  function BeforeEnable() {
    
    //check the settings to see if the plugin has been set up yet. The
    //admin needs to set the gallery directories before the plugin can be
    //enabled.
    global $basepath, $Debuglog, $current_User;
    
    $gallery2path = $this->get_gallery_path();
    
    $this->in_install = true;
    
    //{{{ Search for the user's gallery2 installation.
    $embedphp = $gallery2path."embed.php";
    $mainphp  = $gallery2path."main.php";
    $indexphp = $gallery2path."index.php";
    if (!(file_exists($embedphp) && file_exists($mainphp) && file_exists($indexphp))) {
      $this->msg($this->T_("Gallery2 files not found. Searching for the Gallery2 Path..."));
      
      $g2searchpaths = array(
                             $basepath."gallery2".DIRECTORY_SEPARATOR,
                             $basepath."gallery".DIRECTORY_SEPARATOR,
                             $basepath."..".DIRECTORY_SEPARATOR."gallery2".DIRECTORY_SEPARATOR,
                             $basepath."..".DIRECTORY_SEPARATOR."gallery".DIRECTORY_SEPARATOR
                            );
      
      $gallery2found = false;
      foreach($g2searchpaths as $g2searchpath) {
        $embedphp = $g2searchpath."embed.php";
        $mainphp = $g2searchpath."main.php";
        $indexphp = $g2searchpath."index.php";
        if (file_exists($embedphp) && file_exists($mainphp) && file_exists($indexphp)) {
          $gallery2found = true;
          $this->Settings->set('gallery2path', $this->clean_path($g2searchpath));
          $this->Settings->set('gallery2path_is_absolute', 1);
          break;
        }
      }
      
      if ($gallery2found) {
        $this->msg($this->T_("Found gallery2 in ".$g2searchpath), "success");
        $gallery2path = $g2searchpath;
      } else {
        $this->msg($this->T_("Searched for gallery2 in ".implode(", ", $g2searchpaths).". Please verify your gallery2 installation in the plugin settings."), "error");
        return false;
      }
    }
    //}}}
    
    //{{{ Map users b2evolution <-> gallery2
    
    //init gallery but don't set the login since none of the ids are mapped yet.
    //use the guest login.
    $ret0 = $this->gallery2_init(false, "", "", "", "", $gallery2path);
    
    if (empty($ret0)) {
    
      $g2_image_block_active = 0;
      $g2_moduleid = 'imageblock';
      list ($ret, $g2_modulestatus ) = GalleryCoreApi::fetchPluginStatus('module');
      if (!$ret) {
        if (!isset($g2_modulestatus[$g2_moduleid]) || empty($g2_modulestatus[$g2_moduleid]['active']) || !$g2_modulestatus[$g2_moduleid]['active']) {
          $this->msg($this->T_("Warning: Gallery2 Image Block Plugin has not be activated."), "warning");
        }
      }
      
      $useSSO = $this->Settings->get('useSSO');
      if ($useSSO) {
        
        global $UserCache, $baseurl, $Hit, $localtimenow, $GroupCache, $Settings;
        
        //Sync the existing b2evo users into gallery.
        
        //Get all gallery users. We'll use this later in the second pass.
        //Users added to gallery in the first pass will be ignored.
        list ($ret1, $g2users) = GalleryCoreApi::fetchUserNames();
        // list ($ret2, $idmap) = GalleryEmbed::getExternalIdMap('entityId');
        
        if ($ret1) {
          $this->msg("Could not fetch gallery usernames: ".$ret1->getAsHtml(false), "error");
          return false;
        }
        if (!isset($g2users)) {
          $this->msg("Empty gallery user list!!!", "error");
          return false;
        }
        
        //{{{ first pass will be through the current b2evolution users
        
        //make sure all users are loaded in the cache.
        $UserCache->load_all();
        
        //get all the b2evo users in an array containing their ids
        $idArray = $UserCache->get_ID_array();
        
        foreach($idArray as $id) {
          $b2User = $UserCache->get_by_ID($id);
          
          $Debuglog->add("Checking b2evo user ".$b2User->get('login'));
          
          list ($ret3, $g2_user) = GalleryCoreApi::fetchUserByUsername($b2User->get('login'));
          if ($ret3 && !($ret3->getErrorCode() & ERROR_MISSING_OBJECT)) {
            $this->msg("Could not sync new user with gallery: ".$ret3->getAsHtml(false), "error");
            return false;
          }
          
          if (!isset($g2_user)) {
            $Debuglog->add("no matching gallery2 user: ".$b2User->get('login'));
            // No G2 user with a matching username.  If this is the admin user, we're going to
            // try a little harder and match it to the oldest admin in G2.
            if ($b2User->get('ID') == 1) {
              $Debuglog->add($b2User->get('login')." is the b2evo admin user");
              list ($ret4, $admin_group_id) = GalleryCoreApi::getPluginParameter('module', 'core', 'id.adminGroup');
              if ($ret4) {
                $this->msg("Could not sync new user with gallery: ".$ret4->getAsHtml(false), "error");
                return false;
              }
              
              list ($ret5, $g2_users) = GalleryCoreApi::fetchUsersForGroup($admin_group_id);
              if ($ret5) {
                $this->msg("Could not sync new user with gallery: ".$ret5->getAsHtml(false), "error");
                return false;
              }
              
              $keys = array_keys($g2_users);
              $g2_user_name = $g2_users[$keys[0]];
              list ($ret6, $g2_user) = GalleryCoreApi::fetchUserByUsername($g2_user_name);
              if ($ret6) {
                $this->msg("Could not sync new user with gallery: ".$ret6->getAsHtml(false), "error");
                return false;
              }
            }
          }
          
          if (isset($g2_user)) {
            $Debuglog->add("gallery2 user found: ".$g2_user->getUserName());
            
            // The G2 user was found so add to the External ID Map
            $ret7 = GalleryEmbed::isExternalIdMapped($b2User->get('ID'), 'GalleryUser');
            if ($ret7 && $ret7->getErrorCode() & ERROR_MISSING_OBJECT) {
              $ret8 = GalleryEmbed::addExternalIdMapEntry($b2User->get('ID'), $g2_user->getId(), 'GalleryUser');
              if ($ret8) {
                $Debuglog->add("Could not sync user ".$b2User->get('login')." with gallery: ".$ret8->getAsHtml(false));
                $this->msg("Could not sync user ".$b2User->get('login')." with gallery: ".$ret8->getAsHtml(false), "error");
                return false;
              } else {
                $Debuglog->add("Mapped b2evolution user ".$b2User->get('login')." to gallery user ".$g2_user->getUserName().".");
                $this->msg("Mapped b2evolution user ".$b2User->get('login')." to gallery user ".$g2_user->getUserName().".", "success");
              }
            } else {
              if ($ret7) {
                //Uh oh. A real error happened that we don't know how to handle.
                $Debuglog->add("Could not sync user ".$b2User->get('login')." with gallery: ".$ret7->getAsHtml(false));
                $this->msg("Could not sync user ".$b2User->get('login')." with gallery: ".$ret7->getAsHtml(false), "error");
                return false;
              } else {
                //user is already mapped. OK!
                // $this->msg("User ".$b2User->get('login')." already mapped with gallery.");
                $Debuglog->add("User ".$g2_user->getUserName()." already mapped!");
              }
            }
          } else {
            // No matching G2 user found -- create one.
            $Debuglog->add("Creating gallery2 user: ".$b2User->get('login'));
            $ret8 = GalleryEmbed::createUser($b2User->get('ID'), array('username' => $b2User->get('login'),
                                                                  'email' => $b2User->get('email'),
                                                                  'fullname' => $b2User->get('firstname')." ".$b2User->get('lastname'),
                                                                  'hashedpassword' =>  $b2User->get('pass'),
                                                                  'hashmethod' => "md5",
                                                                  'language' => $b2User->get('locale')));
            if ($ret8) {
              $this->msg($this->T_("An error occurred while checking if a user already exists").": ". $ret8->getAsHtml(false), "error");
              return false;
            } else {
              $this->msg("Created new gallery user ".$b2User->get('login')." for b2evolution user ".$b2User->get('login').".", "success");
            }
          }
          
        }
        //}}}
        
        //{{{ second pass is through the gallery2 users.
        
        //Need to get the external ID map here because we would try to add
        //users who's names match but were not mapped together yet.
        list ($ret2, $idmap) = GalleryEmbed::getExternalIdMap('entityId');
        if ($ret2) {
          $this->msg("Could not fetch gallery ID map: ".$ret2->getAsHtml(false), "error");
          return false;
        }
        
        foreach($g2users as $guserid => $gusername) {
          if (!array_key_exists($guserid, $idmap)) {
            
            list ($ret4, $anonymousUserId) = GalleryCoreApi::getPluginParameter('module', 'core', 'id.anonymousUser');
            if ($ret4) {
              $this->msg("Could not check for anonymous user: ".$ret4->getAsHtml(false), "error");
              return;
            }
            
            //don't create a b2evo login for the anonymous user
            if ($guserid != $anonymousUserId) {
            
              //there is no corresponding user in b2evolution. Need to create
              //it and add a map entry.
              list ($ret3, $g2_user) = GalleryCoreApi::fetchUserByUsername($gusername);
              if ($ret3) {
                $this->msg("Could not fetch gallery userinfo: ".$ret3->getAsHtml(false), "error");
                return false;
              }
              
              $NewUser = new User();
              $NewUser->set('login', $g2_user->getUserName());
              $NewUser->set('nickname', $g2_user->getFullName());
              $NewUser->set('pass', $g2_user->getHashedPassword());
              $NewUser->set('validated', 1 );
              $NewUser->set_email($g2_user->getEmail());
              $language = $g2_user->getLanguage();
              
              $updateLang = empty($language);
              
              if ($updateLang) {
                //language code for b2evolution cannot be NULL
                list($langret, $language) = GalleryTranslator::getDefaultLanguageCode();
                if ($langret) {
                 $this->msg("Unable to get gallery2 default language code: ".$langret->getAsHtml(false), "error");
                 return false;
                }
              }
              
              $NewUser->set('locale', $language);
              $NewUser->set( 'ip', $Hit->IP);
              $NewUser->set( 'domain', $Hit->get_remote_host(true));
              $NewUser->set( 'browser', $Hit->user_agent);
              $NewUser->set_datecreated($localtimenow);
              $NewUser->set( 'level', 1 );
              $NewUser->set( 'notify', 1 );
              $NewUser->set( 'showonline', 1 );
              
              //set the user to the default group for new users
              $newusers_grp_ID = $Settings->get('newusers_grp_ID');
              $new_user_Group = & $GroupCache->get_by_ID( $newusers_grp_ID );
              $NewUser->set_Group( $new_user_Group );
              
              $NewUser->dbinsert();
              $UserCache->add($NewUser);
              $this->msg("Created new b2evolution user ".$NewUser->get('login')." for gallery user ".$g2_user->getUserName().".", "success");
              
              $ret4 = GalleryEmbed::addExternalIdMapEntry($NewUser->get('ID'), $g2_user->getId(), 'GalleryUser');
              if ($ret4) {
                $this->msg("Could not sync new user with gallery: ".$ret4->getAsHtml(false), "error");
                return false;
              } else {
                $this->msg("Mapped b2evolution user ".$NewUser->get('login')." to gallery user ".$g2_user->getUserName().".", "success");
              }
              
              //language code for b2evolution cannot be NULL
              //need to sync language with gallery user since we wrote
              //the current default gallery language to our new b2evo user.
              if ($updateLang) {
                $ret = GalleryEmbed::updateUser($NewUser->get('ID'), array('language' => $b2User->get('locale')));
                $this->msg("Updated language attribute in gallery for ".$NewUser->get('login').".", "success");
              }
            }
          } else {
            //user is already mapped. OK!
            // $this->msg("User ".$gusername." already mapped with b2evolution.");
            $Debuglog->add("gallery2 user ".$gusername." already mapped.");
          }
        }
      //}}}
        
        $ret9 = GalleryEmbed::done();
        if ($ret9) {
           $this->msg("Problem finishing the gallery transaction: ".$ret9->getAsHtml(false), "error");
           return false;
        }
      }
    } else {
      $this->msg("An error occurred while trying to initialize G2: ".$ret0->getAsHtml(false), "error");
      return false;
    }
    //}}}
    
    //{{{ Processing for the gallery2 embedded page
    /*if (!$this->Settings->get('gallery2_page_id') &&
         $this->Settings->get('use_embedded_page') &&
         $this->Settings->get('gallery2_page_category'))
    {
      global $inc_path, $localtimenow;
      
      //This seems like I shouldn't have to do this.
      require_once dirname(__FILE__).'/../../conf/_config.php';
      require_once $inc_path.'items/model/_item.class.php';
      
      //{{{ Create the embedded page.
      $gallery_page_title = "Gallery";
      $gallery_page_content = "This is your gallery2 page. Please don't delete me!! You can change the title of your gallery2 page by changing the title of this post.";
      $post_date = date( 'Y-m-d H:i:s' );
      $post_category = $this->Settings->get('gallery2_page_category');
      
      $Item = & new Item();
      $Item->set( $Item->lasteditor_field, $Item->{$Item->creator_field} );
      $Item->set( 'title', $gallery_page_title );
      $Item->set( 'urltitle', 'gallery' );
      $Item->set( 'content', $gallery_page_content );
      $Item->set( 'datestart', $post_date );
      //Why should I have to do this?
      $Item->set( 'datemodified', date('Y-m-d H:i:s',$localtimenow) );
      $Item->set( 'main_cat_ID', $post_category );
      $Item->set( 'extra_cat_IDs', array() );
      $Item->set( 'status', 'published' );
      //TODO: Fix this to be the default blog locale for the category given.
      $Item->set( 'locale', $current_User->locale );
      $Item->set( 'url', '' );
      $Item->set( 'comment_status', 'disabled' );
      $Item->set_renderers( array($this->code) );
      $Item->set( 'ptyp_ID', 1000 ); //Insert a Page
      $Item->set( 'pst_ID', NULL );
      $Item->set_creator_User( $current_User );
      
      $Item->dbinsert();
      
      $this->Settings->set('gallery2_page_id', $Item->ID);
      //No post processing because we are creating a dummy page.
      
      //}}}
      
      $this->msg(T_("Created gallery2 page successfully."), "success");
    }
    
    if ($this->Settings->get('gallery2_page_id') && !$this->Settings->get('use_embedded_page')) {
      //{{{ Delete the post if it exists.
      $ItemCache = & get_Cache( 'ItemCache' );
      $Item = & $ItemCache->get_by_ID( $this->Settings->get('gallery2_page_id'), false, false );
      if ($Item) {
        
        //If the item exists then delete it.
        $Item->dbdelete();
        
        $this->msg(T_("Deleted gallery2 page."), "success");
      } else {
        $this->msg(T_("The gallery page doesn't exist."));
      }
      //}}}
      
      //Delete the gallery2_page_id no matter what.
      $this->Settings->set('gallery2_page_id', NULL);
    }*/
    //}}}
    
    $this->in_install = false;
    
    //Commit any changed settings to the database
    $this->Settings->dbupdate();
    return true;
  }//}}}
  
  //{{{ GetDefaultSettings()
  
  function GetDefaultSettings() {
    return array('useSSO' => array('label' => $this->T_('Use Single Sign On'),
                                   'type' => 'checkbox',
                                   'defaultvalue' => '0',
                                   'note' => $this->T_('Check this checkbox if you want to use single sign on between your gallery and b2evolution. If you check this checkbox the users in both systems will be synced with eachother.'),
                 ),
                 'gallery2path' => array('label' => $this->T_('Gallery2 Path'),
                                         'size' => 50,
                                         'defaultvalue' => 'gallery2/',
                                         'note' => $this->T_('This is the path to your Gallery2 installation. It can be an absolute path on your server or a relative path from your b2evolution installation directory.'),
                 ),
                 'gallery2path_is_absolute' => array('label' => $this->T_('Absolute Path'),
                                         'type' => 'checkbox',
                                         'defaultvalue' => 0,
                                         'note' => $this->T_('Check this check box if the Gallery2 Path is an absolute path.'),
                 ),
                 'gallery2url' => array('label' => $this->T_('Gallery2 Url'),
                                         'size' => 50,
                                         'defaultvalue' => 'gallery2/',
                                         'note' => $this->T_('This is the url to your Gallery2 installation. It can be an absolute url or a relative url from your b2evolution base url.'),
                 ),
                 'gallery2url_is_absolute' => array('label' => $this->T_('Absolute URL'),
                                         'type' => 'checkbox',
                                         'defaultvalue' => 0,
                                         'note' => $this->T_('Check this check box if the Gallery2 Url is an absolute url.'),
                 ),
                 'embedpath' => array('label' => $this->T_('Embedded Url'),
                                      'size' => 50,
                                      'defaultvalue' => 'gallery/',
                                      'note' => $this->T_('This is the path to your embedded Gallery2 installation. It can be an absolute url or a relative url from your b2evolution base url. This is only required if you use single-sign-on.'),
                 ),
                 'embedpath_is_absolute' => array('label' => $this->T_('Absolute URL'),
                                         'type' => 'checkbox',
                                         'defaultvalue' => 0,
                                         'note' => $this->T_('Check this check box if the Embedded Url is an absolute url.'),
                 ),
                 /*array( 'layout' => 'begin_fieldset', 'label' => $this->T_("Gallery Embedded Page Settings") ),
                 
                 'use_embedded_page' => array('label' => $this->T_('Use Gallery Embedded Page'),
                                                 'type' => 'checkbox',
                                                 'defaultvalue' => '0',
                                                 'note' => $this->T_('Check this checkbox to enable the embedded gallery2 blog page.'),
                 ),
                 
                 'show_sidebar_blocks' => array('label' => $this->T_('Show Sidebar Blocks'),
                                                 'type' => 'checkbox',
                                                 'defaultvalue' => '1',
                                                 'note' => $this->T_('If you are using the Gallery Embedded Page then check this checkbox to show the sidebar panels.'),
                 ),
                 
                 'gallery2_page_category' => array('label' => $this->T_('Gallery2 Page Category'),
                                                 'type' => 'integer',
                                                 'defaultvalue' => 1,
                                                 'note' => $this->T_('This is the category that is assigned to your gallery2 page. This will affect what blog it is placed in and potentially the url to your gallery2 page.'),
                 ),
                 
                 array( 'layout' => 'end_fieldset' ),*/
                 
                 array( 'layout' => 'begin_fieldset', 'label' => $this->T_("G2Image Default Display Settings") ),
                 
                 'g2image_sortby' => array('label' => $this->T_('Sort By'),
                                                    'type' => 'select',
                                                    'options' => array('title_asc' => $this->T_('Gallery2 Title (A-z)'),
                                                                       'title_desc' => $this->T_('Gallery2 Title (z-A)'),
                                                                       'orig_time_desc' => $this->T_('Origination Time (Newest First)'),
                                                                       'orig_time_asc' => $this->T_('Origination Time (Oldest First)'),
                                                                       'mtime_desc' => $this->T_('Last Modification (Newest First)'),
                                                                       'mtime_asc' => $this->T_('Last Modification (Oldest First)')
                                                                      ),
                                                    'defaultvalue' => 'title_asc',
                                                    'note' => $this->T_('The default sorting method for the gallery2 image chooser. Users can override this in their profile.')
                 ),
                 'g2image_images_per_page' => array('label' => $this->T_('Images Per Page'),
                                                    'type' => 'select',
                                                    'options' => array('10' => '10',
                                                                       '15' => '15',
                                                                       '20' => '20',
                                                                       '30' => '30',
                                                                       '40' => '40',
                                                                       '50' => '50',
                                                                       '60' => '60',
                                                                       '9999' => $this->T_('All')
                                                                      ),
                                                    'defaultvalue' => '15',
                                                    'note' => $this->T_('The default number of images per page in the gallery2 image chooser. Users can override this in their profile.')
                  ),
                 'g2image_display_filenames' => array('label' => $this->T_('Display Thumbnails With Info'),
                                                      'type' => 'checkbox',
                                                      'defaultvalue' => '0',
                                                      'note' => $this->T_('The default method of displaying files in the gallery2 image chooser. Users can override this in their profile.')
                 ),
                 'g2image_default_action' => array('label' => $this->T_('How to Insert Image'),
                                   'type' => 'select',
                                   'options' => array('thumbnail_image' => $this->T_('Thumbnail with link to image'),
                                                      'thumbnail_album' => $this->T_('Thumbnail with link to parent album'),
                                                      'thumbnail_lightbox' => $this->T_('Thumbnail with LightBox link to Fullsized Image'),
                                                      'resize_lightbox' => $this->T_('Thumbnail with LightBox link to Resized Image'),
                                                      'thumbnail_custom_url' => $this->T_('Thumbnail with link to custom URL (from text box below)'),
                                                      'thumbnail_only' => $this->T_('Thumbnail only - no link'),
                                                      'fullsize_image' => $this->T_('Fullsized image with link to Gallery page for image'),
                                                      'fullsize_only' => $this->T_('Fullsized image only - no link'),
                                                      'link_image' => $this->T_('Text link to image'),
                                                      'link_album' => $this->T_('Text link to parent album')
                                    ),
                                   'defaultvalue' => 'thumbnail_image',
                                   'note' => $this->T_('The default method of inserting images into posts in the gallery2 image chooser. Users can override this in their profile.')
                 ),
                 
                 array( 'layout' => 'end_fieldset' )
    );
  }//}}}
  
  //{{{ GetDefaultUserSettings()
  /**
   * We allow each user to define their own default settings for G2Image.
   */
  function GetDefaultUserSettings() {
  	 return array('user_g2image_sortby' => array('label' => $this->T_('Sort By'),
                                               'type' => 'select',
                                               'options' => array('title_asc' => $this->T_('Gallery2 Title (A-z)'),
                                                                   'title_desc' => $this->T_('Gallery2 Title (z-A)'),
                                                                   'orig_time_desc' => $this->T_('Origination Time (Newest First)'),
                                                                   'orig_time_asc' => $this->T_('Origination Time (Oldest First)'),
                                                                   'mtime_desc' => $this->T_('Last Modification (Newest First)'),
                                                                   'mtime_asc' => $this->T_('Last Modification (Oldest First)')
                                                                   ),
                                               'defaultvalue' => $this->Settings->get('g2image_sortby'),
                                               'note' => $this->T_('The default sorting method for the gallery2 image chooser. You can override this on a per image basis.')
                                              ),
                 'user_g2image_images_per_page' => array('label' => $this->T_('Images Per Page'),
                                                     'type' => 'select',
                                                     'options' => array('10' => '10',
                                                                         '15' => '15',
                                                                         '20' => '20',
                                                                         '30' => '30',
                                                                         '40' => '40',
                                                                         '50' => '50',
                                                                         '60' => '60',
                                                                         '9999' => $this->T_('All')
                                                                        ),
                                                     'defaultvalue' => $this->Settings->get('g2image_images_per_page'),
                                                     'note' => $this->T_('The default number of images per page in the gallery2 image chooser. You can override this on a per image basis.')
                  ),
                 'user_g2image_display_filenames' => array('label' => $this->T_('Display Thumbnails With Info'),
                                                      'type' => 'checkbox',
                                                      'defaultvalue' => $this->Settings->get('g2image_display_filenames'),
                                                      'note' => $this->T_('The default method of displaying files in the gallery2 image chooser. You can override this on a per image basis.')
                 ),
                 'user_g2image_default_action' => array('label' => $this->T_('How to Insert Image'),
                                   'type' => 'select',
                                   'options' => array('thumbnail_image' => $this->T_('Thumbnail with link to image'),
                                                      'thumbnail_album' => $this->T_('Thumbnail with link to parent album'),
                                                      'thumbnail_lightbox' => $this->T_('Thumbnail with LightBox link to Fullsized Image'),
                                                      'resize_lightbox' => $this->T_('Thumbnail with LightBox link to Resized Image'),
                                                      'thumbnail_custom_url' => $this->T_('Thumbnail with link to custom URL (from text box below)'),
                                                      'thumbnail_only' => $this->T_('Thumbnail only - no link'),
                                                      'fullsize_image' => $this->T_('Fullsized image with link to Gallery page for image'),
                                                      'fullsize_only' => $this->T_('Fullsized image only - no link'),
                                                      'link_image' => $this->T_('Text link to image'),
                                                      'link_album' => $this->T_('Text link to parent album')
                                    ),
                                   'defaultvalue' => $this->Settings->get('g2image_default_action'),
                                   'note' => $this->T_('The default method of inserting images into posts in the gallery2 image chooser. You can override this on a per image basis.')
                 )
    );
  }//}}}
  
  //{{{ AdminDisplayEditorButton()
  /**
	 * Event handler: Called when displaying editor buttons. Adds a button
   * for inserting images from gallery2. Checks if fsckeditor or tinymce
   * plugins are installed and will not display if they are since it doesn't
   * work.
	 *
	 * Note: this gets invoked on all "edit_layout" settings, also "simple",
	 *       because it targets the plugin's audience.
	 *
	 * @param array Associative array of parameters
	 * @return boolean did we display ?
	 */
  function AdminDisplayEditorButton(&$params) {
    global $Plugins;
    $fsckPlugin = $Plugins->get_by_code('evo_fckeditor');
    
    $use_fsckeditor = false;
    if (!empty($fsckPlugin)) {
      $use_fsckeditor = $fsckPlugin->UserSettings->get('use_fsckeditor');
    }
    
    $tinymcePlugin = $Plugins->get_by_classname('tinymce_plugin');
    $use_tinymce = false;
    if (!empty($tinymcePlugin)) {
      $use_tinymce = $tinymcePlugin->UserSettings->get('use_tinymce');
    }
    
    /*
    Don't display the button if we are using fsck_editor or tinymce since
    it doesn't work.
    */
    if (!$use_fsckeditor && !$use_tinymce) {
      echo "<input type=\"button\" name=\"g2image\" value=\"G2Image\" title=\"".$this->T_("Insert a Gallery2 image")."\"onclick=\"g2ic_open('item_checkchanges', 'itemform_post_content');\" />";
      return true;
    }
  }//}}}
  
  //{{{ AdminEndHtmlHead()
  /**
	 * Event handler: Called when ending the admin html head section. This adds
   * some javascript used to open the window for g2image.
	 *
	 * @param array Associative array of parameters
	 * @return boolean did we do something?
	 */
  function AdminEndHtmlHead(&$params) {
		global $ctrl;

		// if( $ctrl != 'edit' ) {
    //   // we're not in the edit screen, so no need to load the JS for AdminDisplayToolbar/AdminDisplayEditorButton
		// 	return false;
		// }
		?>

		<script language="javascript" type="text/javascript">
      function g2ic_open(form, field) {
        var url = '<?php echo $this->get_plugin_url(); ?>g2image/g2image.php?g2ic_form='+form+'&g2ic_field='+field+'&g2ic_tinymce=0';
        var name = 'g2image';
        var w = 600;
        var h = 600;
        var valLeft = (screen.width) ? (screen.width-w)/2 : 0;
        var valTop = (screen.height) ? (screen.height-h)/2 : 0;
        var features = 'width='+w+',height='+h+',left='+valLeft+',top='+valTop+',resizable=1,scrollbars=1';
        window.open(url, name, features);
      }
    </script>

		<?php
		return true;
  }//}}}
  
  //{{{ PluginSettingsValidateSet()
  /**
   * Validates the configuration by making sure the paths given are correct.
   * Basically we check for the /embed.php in the gallery2 path. The embed url
   * is not validated.
   */
  /*function PluginSettingsValidateSet($params) {
    global $basepath;
    
    if ($params["name"] == "gallery2path") {
      $filename = $this->get_gallery_path($params["value"])."main.php";
      
      if (!file_exists($filename)) {
        return $this->T_("Unable to find main.php in the specified gallery2 directory.");
      }
      
      $filename = $this->get_gallery_path($params["value"])."index.php";
      
      if (!file_exists($filename)) {
        return $this->T_("Unable to find index.php in the specified gallery2 directory.");
      }
    }
    
    return NULL;
  }*///}}}
  
  //{{{ AfterUserInsert()
  /**
   * Ensures that users are created in gallery2 when created in b2evolution.
   */
  function AfterUserInsert($params) {
    global $basepath; 
    
    $useSSO = $this->Settings->get('useSSO');
    if ($useSSO && !$this->in_install) {
      
      global $current_User, $baseurl;
      
      $ret = $this->gallery2_init($this->Settings->get('useSSO'));
      if (empty($ret)) {
        
        //now register the new user
        $b2User = $params["User"];
        //make sure the user is not registered into gallery already
        $ret = GalleryEmbed::isExternalIdMapped($b2User->get('ID'), 'GalleryUser');
        if ($ret && $ret->getErrorCode() & ERROR_MISSING_OBJECT) {
          $ret2 = GalleryEmbed::createUser($b2User->get('ID'), array('username' => $b2User->get('login'),
                                                                    'email' => $b2User->get('email'),
                                                                    'fullname' => $b2User->get('firstname')." ".$b2User->get('lastname'),
                                                                    'hashedpassword' =>  $b2User->get('pass'),
                                                                    'hashmethod' => "md5",
                                                                    'language' => $b2User->get('locale')));
          if ($ret2) {
            $this->msg($this->T_("An error occurred while checking if a user already exists").": ". $ret2->getAsHtml(false), "error");
          } else {
            $this->msg("Your user has been added to the gallery.", "success");
          }
        } else {
          if ($ret) {
             /* The error we got wasn't due to a missing user, it was a real error */
             $this->msg($this->T_("An error occurred while checking if a user already exists").": ". $ret->getAsHtml(false), "error");
          }
        }
      } else {
        $this->msg($this->T_("An error occurred while trying to initialize G2").": ".$ret->getAsHtml(false), "error");
      }
      
      $ret = GalleryEmbed::done();
      if ($ret) {
         $this->msg($this->T_("Problem finishing the gallery transaction").": ".$ret->getAsHtml(false), "error");
      }
    }
  }//}}}
  
  //{{{ AfterUserUpdate() 
  /**
   * Ensures that users are synced with gallery2 after update
   */
  function AfterUserUpdate($params) {
    global $basepath; 
    
    $useSSO = $this->Settings->get('useSSO');
    if ($useSSO && !$this->in_install) {
      
      global $current_User, $baseurl;
      
      $ret = $this->gallery2_init($this->Settings->get('useSSO'));
      if (empty($ret)) {
        
        //now register the new user
        $b2User = $params["User"];
        //make sure the user is not registered into gallery already
        $ret2 = GalleryEmbed::isExternalIdMapped($b2User->get('ID'), 'GalleryUser');
        if (empty($ret2)) {
          
          //Gallery chokes if we don't update something. So let's check
          //which values we need to update from the b2User
          
          list ($ret, $g2User) = GalleryCoreApi::loadEntityByExternalId($b2User->get('ID'), 'GalleryUser');
          if ($ret) {
            $this->msg("Could not sync user with gallery: ".$ret->getAsHtml(false), "error");
            return;
          }
          
          //array of fields we need to update
          $dbupdate = array();
          
          if ($g2User->getUserName() != $b2User->get('login')) {
            $dbupdate['username'] = $b2User->get('login');
          }
          
          if ($g2User->getEmail() != $b2User->get('email')) {
            $dbupdate['email'] = $b2User->get('email');
          }
          
          $fullname = $b2User->get('firstname');
          $lastname = $b2User->get('lastname');
          if (!empty($lastname)) {
              $fullname.=" ".$b2User->get('lastname');
          }
          
          if ($g2User->getFullName() != $fullname) {
            $dbupdate['fullname'] = $fullname;
          }
          
          if ($g2User->getHashedPassword() != $b2User->get('pass')) {
            $dbupdate['hashedpassword'] = $b2User->get('pass');
            $dbupdate['hashmethod'] = "md5";
          }
          
          //change the b2evo locale into a gallery language
          $language = $b2User->get('locale');
          $language = str_replace("-", "_", $language);
          if ($g2User->getLanguage() != $language) {
            $dbupdate['language'] = $language;
          }
          
          //$this->msg("Updating username:       ".$b2User->get('login'));
          //$this->msg("Updating email:          ".$b2User->get('email'));
          //$this->msg("Updating fullname:       ".$b2User->get('firstname')." ".$b2User->get('lastname'));
          //$this->msg("Updating hashedpassword: ".$b2User->get('pass'));
          //$this->msg("Updating language:       ".$b2User->get('locale'));
          if (count($dbupdate) > 0) {
            $ret = GalleryEmbed::updateUser($b2User->get('ID'), $dbupdate);
          }
          
          if ($ret) {
             $this->msg("Could not sync user with gallery: ".$ret->getAsHtml(false), "error");
          } else {
            $this->msg("Your profile has been updated with gallery.", "success");
          }
        } else {
           /* The error we got wasn't due to a missing user, it was a real error */
           $this->msg("An error occurred while checking if a user already exists: ". $ret2->getAsHtml(false), "error");
        }
      } else {
        $this->msg("An error occurred while trying to initialize G2: ".$ret->getAsHtml(false), "error");
      }
      
      $ret = GalleryEmbed::done();
      if ($ret) {
         $this->msg("Problem finishing the gallery transaction: ".$ret->getAsHtml(false), "error");
      }
    }
  }//}}}
  
  //{{{ AfterUserDelete()
  /**
   * Ensures that users deleted from b2evolution are deleted in gallery2
   */
  function AfterUserDelete($params) {
    global $basepath;
    
    $useSSO = $this->Settings->get('useSSO');
    if ($useSSO && !$this->in_install) {
      
      global $current_User, $baseurl;
      
      $ret = $this->gallery2_init($this->Settings->get('useSSO'));
      if (empty($ret)) {
        
        //now register the new user
        $b2User = $params["User"];
        //make sure the user is not registered into gallery already
        $ret2 = GalleryEmbed::isExternalIdMapped($b2User->get('ID'), 'GalleryUser');
        if (empty($ret2)) {
          $ret = GalleryEmbed::deleteUser($b2User->get('ID'));
          if ($ret) {
             $this->msg("Could not delete user from gallery: ".$ret->getAsHtml(false), "error");
          } else {
            $this->msg("Deleted user ".$b2User->get('login')." from gallery.", "success");
          }
        } else {
           /* The error we got wasn't due to a missing user, it was a real error */
           $this->msg("An error occurred while checking if a user already exists: ". $ret2->getAsHtml(false), "error");
        }
      } else {
        $this->msg("An error occurred while trying to initialize G2: ".$ret->getAsHtml(false), "error");
      }
      
      $ret = GalleryEmbed::done();
      if ($ret) {
         $this->msg("Problem finishing the gallery transaction: ".$ret->getAsHtml(false), "error");
      }
    }
  }//}}}
  
  //{{{ Logout()
  /**
   * Tells gallery that we logged out so it can clean up session cookies and
   * whatnot.
   */
  function Logout($params) {
    $useSSO = $this->Settings->get('useSSO');
    if ($useSSO) {
      $gallery2path = $this->get_gallery_path();
      
      if (file_exists($gallery2path."embed.php")) {
        require_once($gallery2path."embed.php");
      } else {
        //don't do anything if the plugin is not configured. This shouldn't
        //be called if the plugin is not enabled anyway!!
        $this->msg($this->T_("Gallery2 Single Sign On must be configured before enabling the plugin."), "error");
        return;
      }
      // log the user out using the embed uri.
      GalleryEmbed::logout(array('embedUri' => $this->get_embed_path()));
    }
  }//}}}
  
  //{{{ tinymce_before_init()
  /**
   * Add some code before tinyMCE.init() that will load g2image as
   * an external plugin.
   */
  function tinymce_before_init( &$params ) {
    $g2imageurl = $this->get_plugin_url()."g2image/editor_plugin.js";
    echo "tinymce.PluginManager.load('g2image', '".$g2imageurl."');";
  }//}}}
  
  //{{{ tinymce_extend_plugins()
  /**
   * Tell tinymce not to load g2image during it's normal load sequence because
   * it's already loaded.
   */
  function tinymce_extend_plugins( &$params ) {
    array_push($params["tinymce_plugins"], "-g2image");
  }//}}}
  
  //{{{ tinymce_extend_buttons()
  /**
   * Add the g2image button to the list of buttons in the tinymce toolbar.
   */
  function tinymce_extend_buttons( &$params ) {
    array_push($params["tinymce_buttons"], "g2image");
  }//}}}
  
  // function T_($string, $req_locale = '') {//{{{
  //   //attempt to translate the string using g2image.
  //   $tstring = g2image_T_($string);
  //   if ($string == $tstring) {
  //     return Plugin::T_($string, $req_locale);
  //   } else {
  //     return $tstring;
  //   }
  // }//}}}
  
  //{{{ get_widget_param_definitions()
  /**
   * Get definitions for widget specific editable params
   *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_widget_param_definitions($params) {
		$r = array(
      'title' => array('label' => $this->T_('Custom Heading'),
                                         'size' => 20,
                                         'defaultvalue' => '',
                                         'note' => $this->T_('This is the heading that is displayed at the top of the displayed widget.'),
                 ),
      
      'blocks' => array('label' => $this->T_('Image type'),
                        'type' => 'select',
                        'options' => array("randomImage"  => $this->T_("Random image"),
                                           "recentImage"  => $this->T_("Recent image"),
                                           "viewedImage"  => $this->T_("Viewed image"),
                                           "randomAlbum"  => $this->T_("Random album"),
                                           "recentAlbum"  => $this->T_("Recent album"),
                                           "viewedAlbum"  => $this->T_("Viewed album"),
                                           "dailyImage"   => $this->T_("Daily image"),
                                           "weeklyImage"  => $this->T_("Weekly image"),
                                           "monthyImage"  => $this->T_("Monthy image"),
                                           "dailyAlbum"   => $this->T_("Daily album"),
                                           "weeklyAlbum"  => $this->T_("Weekly album"),
                                           "monthlyAlbum" => $this->T_("Monthly album")
                                           // "specificItem" => $this->T_("Specific item")
                                          ),
                        'defaultvalue' => 'randomImage',
                        'note' => $this->T_('This sets the image type for the gallery2 widget.')
                 ),
      
      'link'            => array('label' => $this->T_("Link URL"),
                                 'type' => 'text',
                                 'size' => 50,
                                 'maxlength' => 255,
                                 'note' => $this->T_('A url to use as the link for the image. Default is to link to the image in the gallery.')
                                ),
      
      'linkTarget'      => array('label' => $this->T_('Link Target'),
                                 'type'  => 'select',
                                 'options' => array(
                                                    "_self" => "Current Window",
                                                    "_blank" => "New Window",
                                                   ),
                                 'defaultvalue' => '_self'
                                ),
      'exactSize'         => array('label' => $this->T_('Thumbnail size'),
                                 'type'  => 'text',
                                 'size'  => 4,
                                 'maxlength' => 4,
                                 'note'  => $this->T_("Sets the size of the thumbnail viewed in the sidebar. "),
                                 'valid_pattern' => array(
                                   "pattern" => "#^[0-9]{0,4}$#s",
                                   "error" => $this->T_("Thumbnail size must be a positive integer or empty.")
                                 )
                             ),
      array( 'layout' => 'begin_fieldset', 'label' => $this->T_("Show") ),
      
      'show_heading'    => array('label' => $this->T_('Heading'),
                                 'type' => 'checkbox',
                                 'defaultvalue' => '0'),
      
      'show_title'      => array('label' => $this->T_('Title'),
                                 'type' => 'checkbox',
                                 'defaultvalue' => '0'),
      'show_date'       => array('label' => $this->T_('Date'),
                                 'type' => 'checkbox',
                                 'defaultvalue' => '0'),
      'show_views'      => array('label' => $this->T_('View Count'),
                                 'type' => 'checkbox',
                                 'defaultvalue' => '0'),
      'show_owner'      => array('label' => $this->T_('Owner'),
                                 'type' => 'checkbox',
                                 'defaultvalue' => '0'),
      
      array( 'layout' => 'end_fieldset')
                                   
		);
		return $r;
  }//}}}
  
  //{{{ SkinTag()
  /**
   * Event handler: SkinTag
   *
   * This is the callback that will be used to display gallery2 widgets like
   * recently added items and random items.
   *
   * We support the following parameters to the block handler.
   *
   * blocks - An array containing blocks to show or a string containing a single
   *          block name
   *          valid values: randomImage, recentImage, viewedImage, randomAlbum,
   *                        recentAlbum, viewedAlbum, dailyImage, weeklyImage,
   *                        monthlyImage, dailyAlbum, weeklyAlbum, monthlyAlbum,
   *                        specificItem
   *
   * show_title - Show the item title.
   *
   * show_heading - Show the item heading.
   *
   * show_date - Show the item date.
   *
   * show_viewcount - show the item view count.
   *
   * show_owner - show the item owner
   *
   * itemId - Limit the item selection to the subtree of the gallery under the
   *          album with the given id; or the id of the item to display when
   *          used with specificItem block type
   *
   * maxSize - Scale images to this maximum size. If used alone Gallery will
   *           locate the most-closely-sized image to the specified value - 
   *           larger images will be scaled down as necessary in your browser.
   *           If specified along with show=fullSize the full size image will
   *           always be used and scaled down as necessary.
   *
   * exactSize - Just like g2_maxSize except that it will not substitute an
   *             image smaller than the size you request, so you'll get the
   *             closest match in size possible. Note that this may use a lot
   *             more bandwidth if a much larger image has to be scaled down in
   *             your browser.
   * 
   * link - Href for link on image; value of none for no link; default is link
   *        to item in the Gallery.
   *
   * linkTarget - Add a link target 
   *              (for example, to open links in a new browser window)
   *
   * @param array Associative array of parameters.
   * @return boolean did we display?
   */
  function SkinTag($params) {
    global $baseurl, $basepath, $Debuglog;
    $output = "";
    
    //{{{ Define valid blocks and properties
    $imageblocks = array(
                          "randomImage",
                          "recentImage",
                          "viewedImage",
                          "randomAlbum",
                          "recentAlbum",
                          "viewedAlbum",
                          "dailyImage",
                          "weeklyImage",
                          "monthyImage",
                          "dailyAlbum",
                          "weeklyAlbum",
                          "monthlyAlbum",
                          "specificItem"
                        );
    $imageproperties = array(
                              "title",
                              "heading",
                              "date",
                              "views",
                              "owner",
                              "fullSize"
                            );
    //}}}
    
    //{{{ Make image blocks into a pipe | separated list. (g2_blocks)
    if (array_key_exists("blocks", $params) && !empty($params["blocks"])) {
      if (!is_array($params['blocks'])) {
        $params["blocks"] = array($params["blocks"]);
      }
      $g2_blocks = array_intersect($params["blocks"], $imageblocks);
      if (count($g2_blocks) > 0) {
        $params["blocks"] = implode("|",$g2_blocks);
      }
    }
    //}}}
    
    //{{{ add property view settings. (g2_show)
    
    //Process which image properties we want to show and create an array
    //of the properties to show
    $showparams = array();
    foreach ($imageproperties as $imageprop) {
      if (array_key_exists("show_".$imageprop,$params) && !empty($params["show_".$imageprop])) {
        if ($params["show_".$imageprop]) {
          $showparams[] = $imageprop;
        }
      }
    }
    
    if (count($showparams) > 0) {
      $params["show"] = implode("|",array_intersect($showparams, $imageproperties));
    } else {
      $params["show"] = "none";
    }//}}}
    
    $ret = $this->gallery2_init($this->Settings->get('useSSO'));
    if (empty($ret)) {
      //getImageBlock() is deprecated in gallery 2.3.0
      if (version_compare($this->g2apiversion, "1.4", ">=")) {
        list ($ret2, $bodyHtml, $headHtml) = GalleryEmbed::getBlock('imageblock', 'ImageBlock', $params);
      } else {
        list ($ret2, $bodyHtml, $headHtml) = GalleryEmbed::getImageBlock($params);
      }
      
      if (empty($ret2)) {
      
        echo $params['block_start'];
        
        echo $params['block_title_start'];
        echo $params["title"];
        echo $params['block_title_end'];
        
        echo $bodyHtml;
        
        echo $params['block_end'];
      }
    } else {
      $this->msg("An error occurred while trying to initialize G2: ".$ret->getAsHtml(false), "error");
    }
    
    $ret = GalleryEmbed::done();
    if ($ret) {
       $this->msg("Problem finishing the gallery transaction: ".$ret->getAsHtml(false), "error");
    }
    
    return true;
    
  }//}}}
  
  //{{{ DisplayItemAsHTML()
  /**
   * Called when displaying an item. This will be used to render the gallery2
   * page if the displayed item is the right ID.
   */
  /*function DisplayItemAsHtml(&$params) {
    //Check if we are displaying the gallery page. This function should only
    //be called for that page but we'll check to make sure.
    if (!$this->admin_page) {
      global $baseurl;
      $item = &$params['Item'];
      $data = &$params['data'];
      if ($item->ID == $this->Settings->get('gallery2_page_id')) {
        $embeduri = $item->get_permanent_url();
        
        $pattern = "%href=[\"\']((?!((.*)://.*|/.*)).*)[\"\']%";
        $replace = 'href="'.substr($embeduri, 0, strrpos($embeduri, "/")+1).'$1"';
        $gallery = $this->runGallery($baseurl, $embeduri);
        $data  = $gallery['javascript'];
        $data .= $gallery['css'];
        $data .= p._replace($pattern, $replace, $gallery['bodyHtml']);
        
        //Stop propagation so that other renderers don't mess up the gallery2 page.
        $this->stop_propagation();
        return true;
      }
    }
    return false;
  }*///}}}
  
  //{{{ ItemApplyAsRenderer()
  /**
	 * Check if we are editing our embedded gallery page and make sure we are
   * included as a renderer.
	 */
	/*function ItemApplyAsRenderer( & $params ) {
		return ($params['Item']->ID == $this->Settings->get('gallery2_page_id'));
  }*///}}}
  
  //{{{ get_embedded_path()
  /**
   * Gets the embedded path for gallery2. This may change depending on if they
   * use the embedded page (the embed page url could changed based on SEO/URL
   * settings). This will be a full url.
   */
  function get_embed_path() {
    global $baseurl;
    
    /*if ($this->Settings->get('use_embedded_page')) {
      $ItemCache = & get_Cache( 'ItemCache' );
      $Item = & $ItemCache->get_by_ID( $this->Settings->get('gallery2_page_id'), false, false );
      if ($Item) {
        return $Item->get_permanent_url();
      }
    }*/
    if ($this->Settings->get('useSSO')) {
      $embedpath = $this->Settings->get('embedpath');
      if ($this->Settings->get('embedpath_is_absolute')) {
        //absolute url
        return $embedpath;
      } else {
        //relative url
        return $this->clean_path($baseurl.$embedpath);
      }
    } else {
      //We aren't using SSO so use the regular gallery install url.
      return $this->get_gallery_url();
    }
  }//}}}
  
  //{{{ get_gallery_path()
  /**
   * Gets the path to the gallery2 installation. This will be an absolute path.
   */
  function get_gallery_path($gallery2path = '', $gallery2path_is_absolute = "") {
    global $basepath;
    
    if ($gallery2path == '') {
      $gallery2path = $this->Settings->get('gallery2path');
    }
    
    if ($gallery2path_is_absolute === "") {
      $gallery2path_is_absolute = $this->Settings->get('gallery2path_is_absolute');
    }
    if ($gallery2path_is_absolute) {
      return $gallery2path;
    } else {
      return $this->clean_path($basepath.$gallery2path);
    }
  }//}}}
  
  //{{{ get_gallery_url()
  /**
   * Gets the url to the gallery2 install. This is used by gallery to generate
   * urls for the images themselves.
   */
  function get_gallery_url() {
    $gallery2url = $this->Settings->get('gallery2url');
    if ($this->Settings->get('gallery2url_is_absolute')) {
      //absolute url
      return $gallery2url;
    } else {
      //relative url
      global $baseurl;
      return $this->clean_path($baseurl.$gallery2url);
    }
  }//}}}
  
  //{{{ clean_path()
  /**
   * Utility function to remove unsightly my/path/here/../../../more/path
   * kinds of urls
   */
  function clean_path($path) {
    $parts = explode(DIRECTORY_SEPARATOR, $path);
    for($i = 1; $i < count($parts); $i++) {
      if ($parts[$i] == "..") {
        array_splice($parts, $i-1, 2);
        $i -= 2;
      }
    }
    //build the path again.
    return implode(DIRECTORY_SEPARATOR, $parts);
  }//}}}
  
  //{{{ gallery2_init()
  
  function gallery2_init($login = true, $baseuri="", $g2uri="", $embeduri="", $redirect="", $gallery2path="") {
    if (!$this->gallery_initialized) {
      global $current_User, $baseurl, $basepath, $current_locale;
      
      // //{{{ Include GalleryEmbed
      if (empty($gallery2path)) {
        $gallery2path = $this->get_gallery_path();
      }
      
      if (file_exists($gallery2path."embed.php")) {
        require_once($gallery2path."embed.php");
      } else {
        //don't do anything if the plugin is not configured. This shouldn't
        //be called if the plugin is not enabled anyway!!
        return new B2G2Error($this->T_("Could not find embed.php in the Gallery2 Path."));
      }//}}}
      
      if (empty($baseuri)) {
        $baseuri = $baseurl;
      }
      if (empty($g2uri)) {
        $g2uri = $this->get_gallery_url();
      }
      if (empty($embeduri)) {
        $embeduri = $this->get_embed_path();
      }
      
      //try to login as the current user.
      $initarray = array('baseUri' => $baseuri,
                         'g2Uri' => $g2uri,
                         'embedUri' => $embeduri,
                         'fullInit' => true,
                         'apiVersion' => $this->minimum_g2apiversion
                        );
      if ($login && isset($current_User)) {
        $initarray['activeUserId'] = $current_User->get('ID');
      } else {
        $initarray['activeUserId'] = '';
      }
      if (isset($current_locale)) {
        $initarray['activeLanguage'] = $current_locale;
      }
      // if (!empty($redirect)) {
      //   $initarray['loginRedirect'] = $redirect;
      // }
      
      $ret = GalleryEmbed::init($initarray);
      if (!$ret) {
        $ver = GalleryEmbed::getApiVersion();
        $this->g2apiversion = $ver[0].".".$ver[1];
      }
      return $ret;
    } else {
      return null;
    }
  }//}}}
  
  //{{{ runGallery()
  /*
  function runGallery($baseuri, $embeduri) {
    global $current_User;
    $gallerypath = $this->Settings->get('gallery2path');
    
    $data = array();
    
    // //Set the login redirect
    // $redirect = $baseuri."htsrv/login.php?redirect_to=".$embeduri;
    
    // initiate G2
    //Make the basepath and embed path the same so that relative urls are
    //handled correctly.
    $ret = $this->gallery2_init($this->Settings->get('useSSO'), "", "", $embeduri);
    if ($ret) {
      $data['bodyHtml'] = $ret->getAsHtml();
      return $data;
    }
    
    // user interface: you could disable sidebar in G2 and get it as separate HTML to put it into a block
    GalleryCapabilities::set('showSidebarBlocks', $this->Settings->get('show_sidebar_blocks'));
    
    GalleryCapabilities::set('login', !$this->Settings->get('useSSO'));
    
    // handle the G2 request
    $g2moddata = GalleryEmbed::handleRequest();
  
    // show error message if isDone is not defined
    if (!isset($g2moddata['isDone'])) {
      $data['bodyHtml'] = 'isDone is not defined, something very bad must have happened.';
      return $data;
    }
    // exit if it was an immediate view / request (G2 already outputted some data)
    if ($g2moddata['isDone']) {
      exit;
    }
   
    // put the body html from G2 into the xaraya template 
    $data['bodyHtml'] = isset($g2moddata['bodyHtml']) ? $g2moddata['bodyHtml'] : '';
  
    // get the page title, javascript and css links from the <head> html from G2
    $title = ''; $javascript = array();	$css = array();
  
    if (isset($g2moddata['headHtml'])) {
      list($data['title'], $css, $javascript) = GalleryEmbed::parseHead($g2moddata['headHtml']);
      $data['headHtml'] = $g2moddata['headHtml'];
    }
    
    
    // Add G2 javascript
    $data['javascript'] = '';
    if (!empty($javascript)) {
      foreach ($javascript as $script) {
        $data['javascript'] .= "\n".$script;
      }
    }
  
    // Add G2 css
    $data['css'] = '';
    if (!empty($css)) {
      foreach ($css as $style) {
        $data['css'] .= "\n".$style;
      }
    }
  
    // sidebar block
    if (isset($g2moddata['sidebarBlocksHtml']) && !empty($g2moddata['sidebarBlocksHtml'])) {
      $data['sidebarHtml'] = $g2moddata['sidebarBlocksHtml'];
    }
      
    return $data;
  }*///}}}
  
}

//{{{ B2G2Error

/**
 * Fake Gallery2 error
 */
class B2G2Error {
	var $msg;
  
  function B2G2Error($msg) {
    $this->msg = $msg;
  }
  
  function getAsHtml($html) {
    return $this->msg;
  }
}//}}}
?>